import org.apache.hadoop.mapred.*;
import org.apache.hadoop.io.*;
import org.apache.hadoop.fs.*;
import org.apache.hadoop.conf.Configuration;

import edu.umass.cs.mallet.base.classify.*;
import edu.umass.cs.mallet.base.pipe.*;
import edu.umass.cs.mallet.base.pipe.iterator.*;
import edu.umass.cs.mallet.base.types.*;
import edu.umass.cs.mallet.base.util.*;

import java.io.*;
import java.util.Date;
import java.util.Random;

public class Project1Mapper extends MapReduceBase implements Mapper {
    
    // a singleton
    static Classifier classifier = null;
    static Pipe instancePipe = null;
    
    public void configure(JobConf job) {
	// if we had coded in our file locations, we would pull them out
	// of the configuration... I'm just going to use a hard coded
	// location
	try {

	    // we could probably have stuck both the classifier and
	    // the alphabet in one file

	    if (classifier == null) {
		// singleton pattern, probably should be synchronized
		Path p = new Path("input/classifier");
		FileSystem fs = p.getFileSystem(job);
		FSDataInputStream fsdis = fs.open(p);

		// load it up from disk
		ObjectInputStream ois = new ObjectInputStream(fsdis);
		classifier = (Classifier)ois.readObject();
		ois.close();

		// grab the instance pipe from the classifier
		instancePipe = classifier.getInstancePipe();
	    }	
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    public void map(WritableComparable key, Writable value,
		    OutputCollector output, Reporter reporter) 
	throws IOException {
	
        // load the instance object, if we had saved it out in the 
	// writable we'd reconstitute it here, but we have text, so...
	Instance inst = 
	    new Instance(value.toString(),key.toString(),
			 key.toString(),instancePipe);

	// only trick is this, we want to pipe the data so that the
	// two pipelines match
	inst = inst.getPipedCopy(instancePipe);

	// generate the classification
       	Classification classification = classifier.classify(inst);

	Text label = 
	    new Text(classification.getLabeling().getBestLabel().toString());
       	
    	// output the filename and classification
	Text target = new Text(key.toString());

	// ship it off
       	output.collect(target,label);
    }

    public static void main(String[] args) throws Exception {
	
	if (args.length == 0) {
	    System.out.println("Usage: Project1Mapper <in-dir> <out-dir>");
	    return;
	}
    
	Path inDir = new Path(args[0]);
	Path outDir = new Path(args[1]);
	
	JobConf job = new JobConf(new Configuration());

	job.setJarByClass(Project1Mapper.class);
	job.setJobName("classifier");
	job.setOutputPath(outDir);
	job.setInputPath(inDir);
	job.setInputFormat(SequenceFileInputFormat.class);

	job.setOutputKeyClass(Text.class);
	job.setOutputValueClass(Text.class);


	job.setNumReduceTasks(1);
	job.setMapperClass(Project1Mapper.class);        
	job.setReducerClass(Project1Reducer.class);
	//	job.setOutputFormat(TextOutputFormat.class);
    
	JobClient client = new JobClient(job);
	ClusterStatus cluster = client.getClusterStatus();
    
	job.setNumMapTasks(20);
	
	Date startTime = new Date();
	System.out.println("Job started: " + startTime);
	JobClient.runJob(job);
	Date endTime = new Date();
	System.out.println("Job ended: " + endTime);
	System.out.println("The job took " + 
			   (endTime.getTime() - startTime.getTime()) /1000 + 
			   " seconds.");

    }
}
