import edu.umass.cs.mallet.base.classify.*;
import edu.umass.cs.mallet.base.pipe.*;
import edu.umass.cs.mallet.base.pipe.iterator.*;
import edu.umass.cs.mallet.base.types.Instance;
import edu.umass.cs.mallet.base.types.InstanceList;
import edu.umass.cs.mallet.base.util.*;

import org.apache.hadoop.io.*;
import org.apache.hadoop.conf.*;
import org.apache.hadoop.fs.*;

import java.io.*;
import java.lang.*;
import java.util.Random;
import java.util.regex.Pattern;

public class VectorBuilder {
	
    public static void main(String[] args) throws Exception {
	
	// Remove common prefix from all the input class directories
	int commonPrefixIndex = Strings.commonPrefixIndex(args);
	// read in the directories, toss out the first common bit of
	// the filename these are our labels
	System.out.println ("Labels = ");
	File[] directories = new File[args.length]; // each class in a
						    // directory
	for (int i = 0; i < args.length; i++) {
	    directories[i] = new File (args[i]);
	    if (commonPrefixIndex < args.length)
		System.out.println ("   "+args[i].substring(commonPrefixIndex));
	    else
		System.out.println ("   "+args[i]);
	}
	
	
	// construct the pipeline
	Pipe instancePipe;
	
	// this is the pipeline to generate Instance objects.  Recall
	// from the documentation that an Instance consists of 4
	// parts: data, target, name, and source (see the javadoc for
	// more info).  The FileIterator which we use to read from the
	// filesystem creates an initial Instance with:
	// data = a pointer to the file
	// target = the path of the file, this is also the label
	// name = the URI to the file
	// source = null
	instancePipe = new SerialPipes (new Pipe[] {

		// the system keeps a "label alphabet" of labels
		// it knows about, so we convert the path to the label, 
		// but basically the system is just building a list 
		// of all the labels
		new Target2Label(), 

		// the system now copies the source from the data field.
		new SaveDataInSource(), 

		// the system looks at the data field, realizes it's a
		// a pointer to a file, loads up the text in the file
		// and stuffs that back into the data field.  Data
		// is now just the text as a big long character array
		new Input2CharSequence(),

		// We now need to split the long character sequence
		// into a token (i.e. word) sequence. So
		// CharSequence2TokenSequence does exactly that.  It
		// replaces the data field of the Instance with a
		// token sequence (a bunch of words)
		new CharSequence2TokenSequence(),

		// At this point you now have the opportunity to run
		// the normalizations you want (lowercasing,
		// etc. etc.)

		// Next, transform the sequence of tokens into a
		// sequence of features (e.g. each word gets assigned
		// a number)
		new TokenSequence2FeatureSequence(),

		// What we're left with in the Instance is
		// data = the feature vector (specifically an object
		// of type FeatureVector)
		// target = the label
		// name = the URI to the file
		// source = the original path
		new FeatureSequence2AugmentableFeatureVector()
	    });
	// read it all in
	InstanceList ilist = new InstanceList(instancePipe);
	boolean removeCommonPrefix=false;

	// the instance list calls the file iterator which produces
	// our starting instance (see above), which then gets passed
	// through the pipeline.  We're going to "cheat" a little and
	// just process the full set of files.  This is the way to go
	// if we want to create one Instance per file to send over to
	// Hadoop, but isn't ideal if we're going to just save the
	// text for the "test" data.  Ideally, we would randomly
	// select a subset here and send 60% of it through the
	// pipeline and keep 40% for later processing.
	ilist.add (new FileIterator(directories,
				    FileIterator.STARTING_DIRECTORIES,
				    removeCommonPrefix));
	

	// lets divide up the data into a train and test set
	InstanceList[] ilists = ilist.split(new Random(), 
					    new double[] {.6,.4});
	InstanceList train = ilists[0];
	InstanceList test = ilists[1];

	// create the trainer
	NaiveBayesTrainer nbt = new NaiveBayesTrainer();
		
	// train the classifier
	Classifier classifier = nbt.train(train);

	// save the classifier to disk
	ObjectOutputStream oos = 
	    new ObjectOutputStream(new FileOutputStream("classifier"));
	oos.writeObject(classifier);
	oos.close();

	// here's where things get a little tricky.  We want to save
	// out the test set somehow.  We have some choices to make.
	// First, is the type of object we're going to keep around for
	// Hadoop.  

	// Keeping around pure instances is generally a bad idea, but
	// you can do that if you want (your file size will blow up).
	// The other option is to make a "light" Instance format which
	// contains the fields necessary to reconstruct the real
	// Instance without all the extra cruft.  This means saving,
	// the target,name, and source (all three of which are small),
	// and also saving the FeatureVector.  We don't want the pure
	// FeatureVector because that contains the alphabet.  Instead
	// we'd grab the indices and values (getIndices() and
	// getValues() respectively). 

	// An easier way to do this, by far, is to just save the text
	// and pass it through the pipeline at each mapper node.

	// Once we've decided if we want to keep around an "Instance"
	// or text we have to decide what file format to use.  If we
	// an Instance, the right thing is probably to use a
	// SequenceFile.  If we use Text we can use either the
	// SequenceFile or stick one file per line in a big text file.
	// I personally think that a SequenceFile is more appropriate
	// for either scheme so I'll walk you through that.

	// create a new configuration object
	Configuration conf = new Configuration();
       
	// we're going to use the local filesystem, and create a local file
	FileSystem fs = FileSystem.getLocal(conf);
	Path p = new Path("test.data");

	// make a new sequence file writer, tell it where to save the
	// file and the type of keys/values.  If we want to use our
	// "light" instance the value type should be ObjectWritable or
	// a BytesWritable (the former means that system takes care of
	// serializing for you, the latter means you have to do it).
	// We're just going to use text/text with no compression (see
	// the sequencefile documentation for information about
	// compression)
	SequenceFile.Writer seq = 
	    SequenceFile.createWriter(fs, conf, p, 
				      Text.class, // the key type 
				      Text.class // the value type
				      );


	// iterate over the instances in the test data
	for (int i = 0; i < test.size(); i++) {
	    Instance inst = test.getInstance(i);

	    Writable value = null;
	    
	    // if we were going to save an Instance in an ObjectWritable
	    // we might do (if we defined the "light instance" type):
	    // LightInstance li = createLightInstance(inst);
	    // value = new ObjectWritable(li);
	    // or just
	    // value = new ObjectWritable(inst) 


	    // If we wanted to use the ByteArrayWritable, we might do:
	    // ByteArrayOutputStream baos = new ByteArrayOutputStream(...)
	    // oos = new ObjectOutputStream(baos)
	    //
	    // oos.writeObject(inst);
	    // or
	    // oos.writeObject(createLightInstance(inst);
	    //
	    // oos.close();
	    // value = new BytesWritable(baos.toByteArray());

	    // But we're just reading text so...
	    BufferedReader br = 
		new BufferedReader(new FileReader(inst.getSource().toString()));
	    StringBuffer content = new StringBuffer();
	    while (br.ready()) {
		content.append(br.readLine());
		content.append('\n');
		// if we wanted to put everything on one line, we'd do
		// content.append(' ') instead
	    }
	    br.close();
	    
	    // the key
	    Text key = new Text(inst.getSource().toString());
	    
	    // the value
	    value = new Text(content.toString());
	    seq.append(key, value);
	}

	// close it up
	seq.close();

	// and we're done... we now have one file
    }
}